/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.mahout.common.iterator;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * <p>
 * An empty {@link Iterator}, which iterates over nothing.
 * </p>
 */
public final class EmptyIterator<T> implements Iterator<T> {
  
  /** @return false */
  @Override
  public boolean hasNext() {
    return false;
  }
  
  /** @return never returns anything */
  @Override
  public T next() {
    throw new NoSuchElementException();
  }
  
  /**
   * @throws UnsupportedOperationException
   */
  @Override
  public void remove() {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public String toString() {
    return "EmptyIterator";
  }
  
}
